////
////  RCB-4搭載のロボットで近距離の物体に近づく
////　簡単な自律プログラムを実行します。
////　KONDOKAGAKU CO.,LTD. 2018.6.22

#include <Rcb4SoftSerialClass.h>
#include <IcsSoftSerialClass.h> 

//---モーション番号---
const byte FORWARD = 1;   //歩行（前）
const byte BACKWARD = 2;  //歩行（後）
const byte LEFTTURN = 3;  //旋回（左）
const byte RIGHTTURN = 4; //旋回（右）
const byte GREETING = 21;  //挨拶

//頭部サーボ検索分岐
const byte LEFT2 = 0;     //左からスタート
const byte LEFT1 = 1;
const byte FRONT = 2;     //頭部旋回角度の中心
const byte RIGHT1 = 3;
const byte RIGHT2 = 4;    //右でストップ

//挨拶モーションを再生する閾値
const int GREETING_POINT = 400;

//頭部シリアルサーボのID番号
const byte HEAD_ID = 0;

//センサを接続した端子番号
const byte AD_CH = 1;

//シールド上のスイッチに配線されている端子番号
const byte SW_PIN = 6;


//-------首旋回範囲-------
const unsigned int SEARCH_STEP = 1250;  //サーボの1回分の移動角度
const unsigned int SEARCH_START = 5000;  //サーボの旋回開始角度


//---センサデータ保管用配列---
unsigned int ad1[5];       //検出したアナログ値
unsigned int adn[5];       //検出方向を保存


//--------RCB4--------
const byte ACK_CHECK_PIN = 3;   //ACKの出力端子を指定
const byte CHECK_PIN = 4;       //通信チェックの端子を指定

const byte S_RX_PIN = 10;     //RCB用送信端子を指定
const byte S_TX_PIN = 11;     //RCB用送信端子を指定

const long BAUDRATE = 115200; //通信速度を指定
const int TIMEOUT_RCB = 500;  //タイムアウトの時間を指定


//--------ICS--------
const byte Sx_RX_PIN = 8;     //ICS用送信端子を指定
const byte Sx_TX_PIN = 9;     //ICS用受信端子を指定

const byte EN_PIN = 2;        //Enable端子を指定
const int TIMEOUT_ICS = 100;  //タイムアウトの時間を指定

//下記の処理でクラスを使えるようにします。
Rcb4SoftSerialClass rcb4(S_RX_PIN,S_TX_PIN,BAUDRATE,TIMEOUT_RCB);  //RCB4Classの定義、softSerial版
IcsSoftSerialClass krs(Sx_RX_PIN,Sx_TX_PIN,EN_PIN,BAUDRATE,TIMEOUT_ICS);  //ICSClassの定義、softSerial版


//----------------------------
//アナログ値平均
//正常な値を検出するために複数回読み取り平均化します。
//----------------------------
//int型の関数のため戻り値の小数点以下は切り捨てます。
int ad_average(char ad_ch){
  int i;
  int adv = 0;
  
  for(i = 0; i < 3; i++){
    delay(10);
    adv += analogRead(ad_ch);   //3回アナログ値を読み取り足していく
  }
  
  return adv/3;                 //3回分のアナログ値を3で割り、戻り値とする
}

//----------------------------
//ソート
//検出したデータを大きい順に入れ替え
//----------------------------
void sort(
  unsigned int t              //検出回数
)
{
  int i, c;
  int tmp;

  //並べ替えるループ
  for(i = 0; i < t-1; i++){
    for(c = i+1; c < t; c++){
      if(ad1[c] > ad1[i]){

        //検出した値用の処理
        tmp = ad1[c];     //大きい値をtmpに一時保管
        ad1[c] = ad1[i];  //小さい値を大きい値の変数に代入
        ad1[i] = tmp;     //一時保管していた大きい値を代入

        //値の検出位置番号を並べ替え
        tmp = adn[c];
        adn[c] = adn[i];
        adn[i] = tmp;
      }
    }
  }
}

//----------------------------
//検出エリア指定しデータ読み込み
//シリアルサーボの頭を動かしながら、データを読み込みます
//----------------------------
void search(
  byte sv_no,                    //頭部サーボのID番号
  byte ad_ch,                    //アナログポートの端子ナンバー
  unsigned int srt_Pos,          //検出スタート位置
  unsigned int stp_Pos,          //検出ステップ
  unsigned int t                 //検索回数
)
{
  int i;
  unsigned int pos1 = 0;

  krs.setPos(sv_no, srt_Pos);         //サーボを検索開始位置に移動
  delay(500);

  for(i = 0; i < t; i++){
    
    pos1 = i * stp_Pos + srt_Pos;    //検出位置を指定
    krs.setPos(sv_no, pos1);         //検出位置へサーボを移動
    delay(200);

    ad1[i] = ad_average(ad_ch);      //読み取ったAD値を代入
//    Serial.print(ad1[i]);          //センサの値をPCに送信
//    Serial.print("  ");

    adn[i] = i;                      //検出位置の番号を代入
  }
  
  krs.setPos(sv_no, 7500);         //頭部を7500（中央）へ移動
  delay(1000);
  
  sort(t);                    //データの大きい順に並べ替え

//  Serial.print(adn[0]);            //最大値の検出番号をPCに送信
}


  
void setup() {
  
  //--------RCB4初期化--------
  pinMode(ACK_CHECK_PIN ,OUTPUT);      //ACK確認を出力するピン設定
  pinMode(CHECK_PIN ,OUTPUT);          //確認を出力するピン設定
  digitalWrite(ACK_CHECK_PIN,LOW);

  rcb4.configData.word = 0xffff;      //ConfigDataは間違えないように0xFFFFで初期化

  while(rcb4.open(115200,1000) == false); //Configをとれないとモーション再生が大変なので、取得できるまでループ
  
  digitalWrite(ACK_CHECK_PIN,HIGH);   //OPENできていたらACKは返ってきている
  
  rcb4.checkAcknowledge();

//--------ICS初期化--------
  krs.begin();                   //サーボモータの通信初期設定

  delay(500);

  krs.setStrc(HEAD_ID, 30);      //頭部サーボのストレッチを30に設定
  krs.setSpd(HEAD_ID, 60);       //頭部サーボのスピードを60に設定

  krs.setPos(HEAD_ID,7500);      //位置指令　頭部サーボを7500（中央）へ
  delay(1000);

  pinMode(5, OUTPUT);            //シールドのLEDにつながっている5番端子を出力に設定
  pinMode(SW_PIN, INPUT);             //シールドのスイッチを入力に設定

//  Serial.begin(9600);          //センサの値を確認する場合はコメントアウトを消す

  while(digitalRead(SW_PIN) == 1){} //スイッチが押されたらloop()スタート
}

void loop() {

  //データを検出し大きい順に並べ替える
  //(サーボID, ADch, 検出スタート位置, 検出ステップ, 検出回数)
  search(HEAD_ID, AD_CH, SEARCH_START, SEARCH_STEP, 5);

  //検出したアナログ値の最大が閾値以上だったら挨拶モーション再生
  if(ad1[0] > GREETING_POINT){
    rcb4.motionPlay(GREETING);
    delay(3000);
  }

  //閾値以下だったら最大値を頼りに移動
  else{
    //最も大きい値を検出した位置番号をもとにモーションを再生
    switch(adn[0]){

     case LEFT2:    //最も左が最大だったとき
      rcb4.motionPlay(LEFTTURN);  //左旋回モーション再生
      delay(2000);                //モーションが終わるまで処理を待つ
      break;

      case LEFT1:   //中央より一つ左が最大だったとき
        rcb4.motionPlay(LEFTTURN);  //左旋回モーション再生
        delay(2000);                //モーションが終わるまで処理を待つ
        break;

      case FRONT:   //中央が最大だったとき
        rcb4.motionPlay(FORWARD);  //前進モーション再生
        delay(2000);               //モーションが終わるまで処理を待つ
        break;

      case RIGHT1:  //中央より一つ右が最大だったとき
        rcb4.motionPlay(RIGHTTURN);  //右旋回モーション再生
        delay(2000);                 //モーションが終わるまで処理を待つ
        break;

      case RIGHT2:  //最も右が最大だったとき
        rcb4.motionPlay(RIGHTTURN);  //右旋回モーション再生
        delay(2000);                 //モーションが終わるまで処理を待つ
        break;

      default:      //どの条件とも会わなかったとき
        digitalWrite(5, HIGH);        //シールド上のLED点灯
        delay(1000);
        digitalWrite(5, LOW);        //シールド上のLED消灯
        break;
    }
  }
}

